@@ -1,4 +1,5 @@
 /*
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *  Author: Tomasz Kojm <tkojm@clamav.net>
  *
@@ -35,10 +36,6 @@
 #include <sys/stat.h>
 #include <dirent.h>
 
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-#include "libclamav/crypto.h"
-
 #include "shared/optparser.h"
 #include "shared/misc.h"
 
@@ -89,7 +86,7 @@ static void printopts(struct optstruct *opts, int nondef)
 	if(!opts->enabled) 
 	    printf("%s disabled\n", opts->name);
 	else switch(clam_options[opts->idx].argtype) {
-	    case TYPE_STRING:
+	    case CLOPT_TYPE_STRING:
 		printf("%s = \"%s\"", opts->name, opts->strarg);
 		opt = opts;
 		while((opt = opt->nextarg))
@@ -97,8 +94,8 @@ static void printopts(struct optstruct *opts, int nondef)
 		printf("\n");
 		break;
 
-	    case TYPE_NUMBER:
-	    case TYPE_SIZE:
+	    case CLOPT_TYPE_NUMBER:
+	    case CLOPT_TYPE_SIZE:
 		printf("%s = \"%lld\"", opts->name, opts->numarg);
 		opt = opts;
 		while((opt = opt->nextarg))
@@ -106,7 +103,7 @@ static void printopts(struct optstruct *opts, int nondef)
 		printf("\n");
 		break;
 
-	    case TYPE_BOOL:
+	    case CLOPT_TYPE_BOOL:
 		printf("%s = \"yes\"\n", opts->name);
 		break;
 
@@ -152,21 +149,21 @@ static int printconf(const char *name)
 		printf("# %s\n", tokens[j]);
 
 	    switch(cpt->argtype) {
-		case TYPE_STRING:
+		case CLOPT_TYPE_STRING:
 		    if(cpt->strarg)
 			printf("# Default: %s\n", cpt->strarg);
 		    else
 			printf("# Default: disabled\n");
 		    break;
 
-		case TYPE_NUMBER:
+		case CLOPT_TYPE_NUMBER:
 		    if(cpt->numarg != -1)
 			printf("# Default: %lld\n", cpt->numarg);
 		    else
 			printf("# Default: disabled\n");
 		    break;
 
-		case TYPE_SIZE:
+		case CLOPT_TYPE_SIZE:
 		    printf("# You may use 'M' or 'm' for megabytes (1M = 1m = 1048576 bytes)\n# and 'K' or 'k' for kilobytes (1K = 1k = 1024 bytes). To specify the size\n# in bytes just don't use modifiers.\n");
 		    if(cpt->numarg != -1)
 			printf("# Default: %lld\n", cpt->numarg);
@@ -174,7 +171,7 @@ static int printconf(const char *name)
 			printf("# Default: disabled\n");
 		    break;
 
-		case TYPE_BOOL:
+		case CLOPT_TYPE_BOOL:
 		    if(cpt->numarg != -1)
 			printf("# Default: %s\n", cpt->numarg ? "yes" : "no");
 		    else
@@ -204,8 +201,8 @@ static void help(void)
 {
     printf("\n");
     printf("           Clam AntiVirus: Configuration Tool %s\n", get_version());
-    printf("           By The ClamAV Team: http://www.clamav.net/team\n");
-    printf("           (C) 2009 Sourcefire, Inc.\n\n");
+    printf("           By The ClamAV Team: http://www.clamav.net/about.html#credits\n");
+    printf("           (C) 2009-2015 Cisco Systems, Inc.\n\n");
 
     printf("    --help                 -h         Show help\n");
     printf("    --version              -V         Show version\n");
@@ -256,6 +253,7 @@ static void print_platform(struct cli_environment *env)
     printf("zlib version: %s (%s)\n",
 	   ZLIB_VERSION, zlibVersion());
 #endif
+
     if (env->triple[0])
     printf("Triple: %s\n", env->triple);
     if (env->cpu[0])
@@ -376,8 +374,6 @@ int main(int argc, char **argv)
 	unsigned int i, j;
 	struct cli_environment env;
 
-    cl_initialize_crypto();
-
     opts = optparse(NULL, argc, argv, 1, OPT_CLAMCONF, 0, NULL);
     if(!opts) {
 	printf("ERROR: Can't parse command line options\n");
@@ -462,6 +458,19 @@ int main(int argc, char **argv)
 #ifdef HAVE_BZLIB_H
 	printf("BZIP2 ");
 #endif
+
+#ifdef HAVE_LIBXML2
+	printf("LIBXML2 ");
+#endif
+#ifdef HAVE_PCRE
+	printf("PCRE ");
+#endif
+#ifdef HAVE_ICONV
+	printf("ICONV ");
+#endif
+#ifdef HAVE_JSON
+	printf("JSON ");
+#endif
     if(have_rar)
 	printf("RAR ");
     if (have_clamjit)
