@@ -1,4 +1,5 @@
 /*
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2009 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -53,10 +54,6 @@
 #include <sys/resource.h>
 #endif
 
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-#include "libclamav/crypto.h"
-
 #include "target.h"
 
 #include "libclamav/clamav.h"
@@ -80,13 +77,14 @@ short foreground = 0;
 char hostid[37];
 
 char *get_hostid(void *cbdata);
+int is_valid_hostid(void);
 
 static void help(void)
 {
     printf("\n");
     printf("                      Clam AntiVirus Daemon %s\n", get_version());
-    printf("           By The ClamAV Team: http://www.clamav.net/team\n");
-    printf("           (C) 2007-2009 Sourcefire, Inc.\n\n");
+    printf("           By The ClamAV Team: http://www.clamav.net/about.html#credits\n");
+    printf("           (C) 2007-2015 Cisco Systems, Inc.\n\n");
 
     printf("    --help                   -h             Show this help.\n");
     printf("    --version                -V             Show version number.\n");
@@ -122,11 +120,12 @@ int main(int argc, char **argv)
     time_t currtime;
     const char *dbdir, *cfgfile;
     char *pua_cats = NULL, *pt;
-    int ret, tcpsock = 0, localsock = 0, i, min_port, max_port;
+    int ret, tcpsock = 0, localsock = 0, min_port, max_port;
     unsigned int sigs = 0;
     int *lsockets=NULL;
     unsigned int nlsockets = 0;
     unsigned int dboptions = 0;
+    unsigned int i;
 #ifdef C_LINUX
     STATBUF sb;
 #endif
@@ -139,8 +138,6 @@ int main(int argc, char **argv)
     sa.sa_handler = SIG_IGN;
     sigaction(SIGHUP, &sa, NULL);
     sigaction(SIGUSR2, &sa, NULL);
-#else
-    cl_initialize_crypto();
 #endif
 
     if((opts = optparse(NULL, argc, argv, 1, OPT_CLAMD, 0, NULL)) == NULL) {
@@ -167,6 +164,10 @@ int main(int argc, char **argv)
     /* parse the config file */
     cfgfile = optget(opts, "config-file")->strarg;
     pt = strdup(cfgfile);
+    if (pt == NULL) {
+	fprintf(stderr, "ERROR: Unable to allocate memory for config file\n");
+	return 1;
+    }
     if((opts = optparse(cfgfile, 0, NULL, 1, OPT_CLAMD, 0, opts)) == NULL) {
         fprintf(stderr, "ERROR: Can't open/parse the config file %s\n", pt);
         free(pt);
@@ -488,6 +489,8 @@ int main(int argc, char **argv)
 
         cl_engine_set_clcb_hash(engine, hash_callback);
 
+        cl_engine_set_clcb_virus_found(engine, clamd_virus_found_cb);
+
         if(optget(opts, "LeaveTemporaryFiles")->enabled)
             cl_engine_set_num(engine, CL_ENGINE_KEEPTMP, 1);
 
@@ -570,11 +573,34 @@ int main(int argc, char **argv)
             break;
         }
 
+        if((ret = statinidir_th(dbdir))) {
+            logg("!%s\n", cl_strerror(ret));
+            ret = 1;
+            break;
+        }
+
         if (optget(opts, "DisableCertCheck")->enabled)
             engine->dconf->pe |= PE_CONF_DISABLECERT;
 
         logg("#Loaded %u signatures.\n", sigs);
 
+        /* pcre engine limits - required for cl_engine_compile */
+        if((opt = optget(opts, "PCREMatchLimit"))->active) {
+            if((ret = cl_engine_set_num(engine, CL_ENGINE_PCRE_MATCH_LIMIT, opt->numarg))) {
+                logg("!cli_engine_set_num(PCREMatchLimit) failed: %s\n", cl_strerror(ret));
+                cl_engine_free(engine);
+                return 1;
+            }
+        }
+
+        if((opt = optget(opts, "PCRERecMatchLimit"))->active) {
+            if((ret = cl_engine_set_num(engine, CL_ENGINE_PCRE_RECMATCH_LIMIT, opt->numarg))) {
+                logg("!cli_engine_set_num(PCRERecMatchLimit) failed: %s\n", cl_strerror(ret));
+                cl_engine_free(engine);
+                return 1;
+            }
+        }
+
         if((ret = cl_engine_compile(engine)) != 0) {
             logg("!Database initialization error: %s\n", cl_strerror(ret));
             ret = 1;
@@ -582,8 +608,6 @@ int main(int argc, char **argv)
         }
 
         if(tcpsock) {
-            int *t;
-
             opt = optget(opts, "TCPAddr");
             if (opt->enabled) {
                 int breakout = 0;
@@ -676,7 +700,7 @@ int main(int argc, char **argv)
         if(!optget(opts, "Foreground")->enabled) {
 #ifdef C_BSD	    
             /* workaround for OpenBSD bug, see https://wwws.clamav.net/bugzilla/show_bug.cgi?id=885 */
-            for(ret=0;ret<nlsockets;ret++) {
+            for(ret=0;(unsigned int)ret<nlsockets;ret++) {
                 if (fcntl(lsockets[ret], F_SETFL, fcntl(lsockets[ret], F_GETFL) | O_NONBLOCK) == -1) {
                     logg("!fcntl for lsockets[] failed\n");
                     close(lsockets[ret]);
@@ -694,7 +718,7 @@ int main(int argc, char **argv)
             }
             gengine = NULL;
 #ifdef C_BSD
-            for(ret=0;ret<nlsockets;ret++) {
+            for(ret=0;(unsigned int)ret<nlsockets;ret++) {
                 if (fcntl(lsockets[ret], F_SETFL, fcntl(lsockets[ret], F_GETFL) & ~O_NONBLOCK) == -1) {
                     logg("!fcntl for lsockets[] failed\n");
                     close(lsockets[ret]);
@@ -744,6 +768,8 @@ int main(int argc, char **argv)
     logg_close();
     optfree(opts);
 
+    cl_cleanup_crypto();
+
     return ret;
 }
 
@@ -770,6 +796,8 @@ int is_valid_hostid(void)
 
 char *get_hostid(void *cbdata)
 {
+    UNUSEDPARAM(cbdata);
+
     if (!strcmp(hostid, "none"))
         return NULL;
 
