@@ -1,4 +1,5 @@
 /*
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2009 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm, Török Edvin
@@ -43,6 +44,8 @@
 #define CMD19 "DETSTATSCLEAR"
 #define CMD20 "DETSTATS"
 
+#define CMD21 "ALLMATCHSCAN"
+
 #include "libclamav/clamav.h"
 #include "shared/optparser.h"
 #include "server.h"
@@ -70,7 +73,8 @@ enum commands {
     COMMAND_DETSTATS,
     /* internal commands */
     COMMAND_MULTISCANFILE,
-    COMMAND_INSTREAMSCAN
+    COMMAND_INSTREAMSCAN,
+    COMMAND_ALLMATCHSCAN
 };
 
 typedef struct client_conn_tag {
@@ -96,7 +100,7 @@ int execute_or_dispatch_command(client_conn_t *conn, enum commands command, cons
 
 int conn_reply(const client_conn_t *conn, const char *path, const char *msg, const char *status);
 int conn_reply_single(const client_conn_t *conn, const char *path, const char *status);
-int conn_reply_virus(const client_conn_t *conn, const char *file, const char *virname, const char *virhash, unsigned int virsize);
+int conn_reply_virus(const client_conn_t *conn, const char *file, const char *virname);
 int conn_reply_error(const client_conn_t *conn, const char *msg);
 int conn_reply_errno(const client_conn_t *conn, const char *path, const char *msg);
 #endif
