@@ -46,10 +46,6 @@
 #include <syslog.h>
 #endif
 
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-#include "libclamav/crypto.h"
-
 #include "target.h"
 #include "clamav.h"
 #include "freshclamcodes.h"
@@ -76,6 +72,7 @@ char hostid[37];
 
 void submit_host_info(struct optstruct *opts);
 char *get_hostid(void *cbdata);
+int is_valid_hostid(void);
 
 static void
 sighandler (int sig)
@@ -138,7 +135,7 @@ writepid (const char *pidfile)
     }
     else
     {
-        fprintf (fd, "%d", (int) getpid ());
+        fprintf (fd, "%d\n", (int) getpid ());
         fclose (fd);
     }
     umask (old_umask);
@@ -152,8 +149,8 @@ help (void)
     mprintf ("\n");
     mprintf ("                   Clam AntiVirus: freshclam  %s\n",
              get_version ());
-    printf ("           By The ClamAV Team: http://www.clamav.net/team\n");
-    printf ("           (C) 2007-2009 Sourcefire, Inc. et al.\n\n");
+    printf ("           By The ClamAV Team: http://www.clamav.net/about.html#credits\n");
+    printf ("           (C) 2007-2015 Cisco Systems, Inc.\n\n");
 
     mprintf ("    --help               -h              show help\n");
     mprintf
@@ -167,6 +164,8 @@ help (void)
         ("    --no-warnings                        don't print and log warnings\n");
     mprintf
         ("    --stdout                             write to stdout instead of stderr\n");
+    mprintf
+        ("    --show-progress                      show download progress percentage\n");
     mprintf ("\n");
     mprintf
         ("    --config-file=FILE                   read configuration from FILE.\n");
@@ -247,7 +246,7 @@ download (const struct optstruct *opts, const char *cfgfile)
                     opt = (struct optstruct *) opt->nextarg;
                     if (!opt)
                     {
-                        logg ("Update failed. Your network may be down or none of the mirrors listed in %s is working. Check http://www.clamav.net/support/mirror-problem for possible reasons.\n", cfgfile);
+                        logg ("Update failed. Your network may be down or none of the mirrors listed in %s is working. Check http://www.clamav.net/doc/mirrors-faq.html for possible reasons.\n", cfgfile);
                     }
                 }
 
@@ -266,6 +265,9 @@ static void
 msg_callback (enum cl_msg severity, const char *fullmsg, const char *msg,
               void *ctx)
 {
+    UNUSEDPARAM(fullmsg);
+    UNUSEDPARAM(ctx);
+
     switch (severity)
     {
     case CL_MSG_ERROR:
@@ -302,8 +304,6 @@ main (int argc, char **argv)
     if (check_flevel ())
         exit (FCE_INIT);
 
-    cl_initialize_crypto();
-
     if ((retcl = cl_init (CL_INIT_DEFAULT)))
     {
         mprintf ("!Can't initialize libclamav: %s\n", cl_strerror (retcl));
@@ -458,6 +458,9 @@ main (int argc, char **argv)
     if (optget (opts, "stdout")->enabled)
         mprintf_stdout = 1;
 
+    if (optget (opts, "show-progress")->enabled)
+        mprintf_progress = 1;
+
     /* initialize logger */
     logg_verbose = mprintf_verbose ? 1 : optget (opts, "LogVerbose")->enabled;
     logg_time = optget (opts, "LogTime")->enabled;
@@ -748,7 +751,6 @@ main (int argc, char **argv)
 
 void submit_host_info(struct optstruct *opts)
 {
-    struct optstruct *opt;
     struct cl_engine *engine;
     cli_intel_t *intel;
 
@@ -818,6 +820,8 @@ int is_valid_hostid(void)
 
 char *get_hostid(void *cbdata)
 {
+    UNUSEDPARAM(cbdata);
+
     if (!strcmp(hostid, "none"))
         return NULL;
 
