@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2007-2008 Sourcefire, Inc.
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
  *
@@ -40,6 +41,8 @@ struct cli_dconf {
     uint32_t other;
     uint32_t phishing;
     uint32_t bytecode;
+    uint32_t stats;
+    uint32_t pcre;
 };
 
 /* PE flags */
@@ -62,6 +65,7 @@ struct cli_dconf {
 #define PE_CONF_CATALOG	    0x10000
 #define PE_CONF_DISABLECERT 0x20000
 #define PE_CONF_DUMPCERT    0x40000
+#define PE_CONF_MATCHICON   0x80000
 
 /* Archive flags */
 #define ARCH_CONF_RAR	    0x1
@@ -86,6 +90,10 @@ struct cli_dconf {
 #define ARCH_CONF_XAR       0x80000
 #define ARCH_CONF_HFSPLUS   0x100000
 #define ARCH_CONF_XZ        0x200000
+#define ARCH_CONF_PASSWD    0x400000
+#define ARCH_CONF_MBR       0x800000
+#define ARCH_CONF_GPT       0x1000000
+#define ARCH_CONF_APM       0x2000000
 
 /* Document flags */
 #define DOC_CONF_HTML		0x1
@@ -95,6 +103,9 @@ struct cli_dconf {
 #define DOC_CONF_HTML_SKIPRAW	0x10
 #define DOC_CONF_JSNORM         0x20
 #define DOC_CONF_SWF		0x40
+#define DOC_CONF_MSXML		0x80
+#define DOC_CONF_OOXML		0x100
+#define DOC_CONF_HWP		0x200
 
 /* Mail flags */
 #define MAIL_CONF_MBOX	    0x1
@@ -110,6 +121,7 @@ struct cli_dconf {
 #define OTHER_CONF_MYDOOMLOG	0x40
 #define OTHER_CONF_PREFILTERING 0x80
 #define OTHER_CONF_PDFNAMEOBJ	0x100
+#define OTHER_CONF_PRTNINTXN	0x200
 
 /* Phishing flags */
 #define PHISHING_CONF_ENGINE   0x1
@@ -121,6 +133,15 @@ struct cli_dconf {
 #define BYTECODE_JIT_PPC 0x4
 #define BYTECODE_JIT_ARM 0x8
 
+/* Stats/Intel flags */
+#define DCONF_STATS_DISABLED            0x1
+#define DCONF_STATS_PE_SECTION_DISABLED 0x2
+
+/* PCRE flags */
+#define PCRE_CONF_SUPPORT 0x1
+#define PCRE_CONF_OPTIONS 0x2
+#define PCRE_CONF_GLOBAL  0x4
+
 #define BYTECODE_ENGINE_MASK (BYTECODE_INTERPRETER | BYTECODE_JIT_X86 | BYTECODE_JIT_PPC | BYTECODE_JIT_ARM)
 
 #ifdef USE_MPOOL
