@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2007-2010 Sourcefire, Inc.
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
  *
@@ -50,6 +51,18 @@
 #include "events.h"
 #include "crtmgr.h"
 
+#ifdef HAVE_JSON
+#include "json.h"
+#endif
+
+#ifdef HAVE_YARA
+#include "yara_clam.h"
+#endif
+
+#if HAVE_LIBXML2
+#define CLAMAV_MIN_XMLREADER_FLAGS (XML_PARSE_NOERROR | XML_PARSE_NONET)
+#endif
+
 /*
  * CL_FLEVEL is the signature f-level specific to the current code and
  *	     should never be modified
@@ -59,7 +72,7 @@
  * in re-enabling affected modules.
  */
 
-#define CL_FLEVEL 77
+#define CL_FLEVEL 82
 #define CL_FLEVEL_DCONF	CL_FLEVEL
 #define CL_FLEVEL_SIGTOOL CL_FLEVEL
 
@@ -116,8 +129,7 @@ typedef struct bitset_tag
 /* internal clamav context */
 typedef struct cli_ctx_tag {
     const char **virname;
-    unsigned int num_viruses;         /* manages virname when CL_SCAN_ALLMATCHES == 1 */
-    unsigned int size_viruses;        /* manages virname when CL_SCAN_ALLMATCHES == 1 */
+    unsigned int num_viruses;
     unsigned long int *scanned;
     const struct cli_matcher *root;
     const struct cl_engine *engine;
@@ -140,6 +152,11 @@ typedef struct cli_ctx_tag {
     char entry_filename[2048];
     int sha_collect;
 #endif
+#ifdef HAVE_JSON
+    struct json_object *properties;
+    struct json_object *wrkproperty;
+#endif
+    struct timeval time_limit;
 } cli_ctx;
 
 #define STATS_ANON_UUID "5b585e8f-3be5-11e3-bf0b-18037319526c"
@@ -149,7 +166,7 @@ typedef struct cli_ctx_tag {
 typedef struct cli_flagged_sample {
     char **virus_name;
     char md5[16];
-    size_t size; /* A size of zero means size is unavailable (why would this ever happen?) */
+    uint32_t size; /* A size of zero means size is unavailable (why would this ever happen?) */
     uint32_t hits;
     stats_section_t *sections;
 
@@ -216,6 +233,20 @@ struct cli_dbinfo {
     struct cli_dbinfo *next;
 };
 
+#define CLI_PWDB_COUNT 3
+typedef enum {
+    CLI_PWDB_ANY = 0,
+    CLI_PWDB_ZIP = 1,
+    CLI_PWDB_RAR = 2
+} cl_pwdb_t;
+
+struct cli_pwdb {
+    char *name;
+    unsigned char *passwd;
+    uint16_t length;
+    struct cli_pwdb *next;
+};
+
 struct cl_engine {
     uint32_t refcount; /* reference counter */
     uint32_t sdb;
@@ -272,6 +303,9 @@ struct cl_engine {
     struct cli_ftype *ftypes;
     struct cli_ftype *ptypes;
 
+    /* Container password storage */
+    struct cli_pwdb **pwdbs;
+
     /* Ignored signatures */
     struct cli_matcher *ignored;
 
@@ -297,10 +331,12 @@ struct cl_engine {
     clcb_pre_cache cb_pre_cache;
     clcb_pre_scan cb_pre_scan;
     clcb_post_scan cb_post_scan;
+    clcb_virus_found cb_virus_found;
     clcb_sigload cb_sigload;
     void *cb_sigload_ctx;
     clcb_hash cb_hash;
     clcb_meta cb_meta;
+    clcb_file_props cb_file_props;
 
     /* Used for bytecode */
     struct cli_all_bc bcs;
@@ -330,10 +366,24 @@ struct cl_engine {
     clcb_stats_get_hostid cb_stats_get_hostid;
 
     /* Raw disk image max settings */
-    uint32_t maxpartitions;
+    uint32_t maxpartitions; /* max number of partitions to scan in a disk image */
 
     /* Engine max settings */
     uint32_t maxiconspe; /* max number of icons to scan for PE */
+    uint32_t maxrechwp3; /* max recursive calls for HWP3 parsing */
+
+    /* millisecond time limit for preclassification scanning */
+    uint32_t time_limit;
+
+    /* PCRE matching limitations */
+    uint64_t pcre_match_limit;
+    uint64_t pcre_recmatch_limit;
+    uint64_t pcre_max_filesize;
+
+#ifdef HAVE_YARA
+    /* YARA */
+    struct _yara_global * yara_global;
+#endif
 };
 
 struct cl_settings {
@@ -362,11 +412,13 @@ struct cl_settings {
     clcb_pre_cache cb_pre_cache;
     clcb_pre_scan cb_pre_scan;
     clcb_post_scan cb_post_scan;
+    clcb_virus_found cb_virus_found;
     clcb_sigload cb_sigload;
     void *cb_sigload_ctx;
     clcb_msg cb_msg;
     clcb_hash cb_hash;
     clcb_meta cb_meta;
+    clcb_file_props cb_file_props;
 
     /* Engine max settings */
     uint64_t maxembeddedpe;  /* max size to scan MSEXE for PE */
@@ -391,6 +443,12 @@ struct cl_settings {
 
     /* Engine max settings */
     uint32_t maxiconspe; /* max number of icons to scan for PE */
+    uint32_t maxrechwp3; /* max recursive calls for HWP3 parsing */
+
+    /* PCRE matching limitations */
+    uint64_t pcre_match_limit;
+    uint64_t pcre_recmatch_limit;
+    uint64_t pcre_max_filesize;
 };
 
 extern int (*cli_unrar_open)(int fd, const char *dirname, unrar_state_t *state);
@@ -414,6 +472,9 @@ extern int have_rar;
 #define SCAN_STRUCTURED	    (ctx->options & CL_SCAN_STRUCTURED)
 #define SCAN_ALL            (ctx->options & CL_SCAN_ALLMATCHES)
 #define SCAN_SWF            (ctx->options & CL_SCAN_SWF)
+#define SCAN_PROPERTIES     (ctx->options & CL_SCAN_FILE_PROPERTIES)
+#define SCAN_XMLDOCS        (ctx->options & CL_SCAN_XMLDOCS)
+#define SCAN_HWP3           (ctx->options & CL_SCAN_HWP3)
 
 /* based on macros from A. Melnikoff */
 #define cbswap16(v) (((v & 0xff) << 8) | (((v) >> 8) & 0xff))
@@ -485,30 +546,30 @@ struct unaligned_ptr {
 #define be32_to_host(v)	(v)
 #define be64_to_host(v)	(v)
 
-static inline int32_t cli_readint32(const char *buff)
+static inline int32_t cli_readint32(const void *buff)
 {
 	int32_t ret;
-    ret = buff[0] & 0xff;
-    ret |= (buff[1] & 0xff) << 8;
-    ret |= (buff[2] & 0xff) << 16;
-    ret |= (buff[3] & 0xff) << 24;
+    ret = ((const char *)buff)[0] & 0xff;
+    ret |= (((const char *)buff)[1] & 0xff) << 8;
+    ret |= (((const char *)buff)[2] & 0xff) << 16;
+    ret |= (((const char *)buff)[3] & 0xff) << 24;
     return ret;
 }
 
-static inline int16_t cli_readint16(const char *buff)
+static inline int16_t cli_readint16(const void *buff)
 {
 	int16_t ret;
-    ret = buff[0] & 0xff;
-    ret |= (buff[1] & 0xff) << 8;
+    ret = ((const char *)buff)[0] & 0xff;
+    ret |= (((const char *)buff)[1] & 0xff) << 8;
     return ret;
 }
 
-static inline void cli_writeint32(char *offset, uint32_t value)
+static inline void cli_writeint32(void *offset, uint32_t value)
 {
-    offset[0] = value & 0xff;
-    offset[1] = (value & 0xff00) >> 8;
-    offset[2] = (value & 0xff0000) >> 16;
-    offset[3] = (value & 0xff000000) >> 24;
+    ((char *)offset)[0] = value & 0xff;
+    ((char *)offset)[1] = (value & 0xff00) >> 8;
+    ((char *)offset)[2] = (value & 0xff0000) >> 16;
+    ((char *)offset)[3] = (value & 0xff000000) >> 24;
 }
 #endif
 
@@ -632,6 +693,8 @@ int cli_updatelimits(cli_ctx *, unsigned long);
 unsigned long cli_getsizelimit(cli_ctx *, unsigned long);
 int cli_matchregex(const char *str, const char *regex);
 void cli_qsort(void *a, size_t n, size_t es, int (*cmp)(const void *, const void *));
+void cli_qsort_r(void *a, size_t n, size_t es, int (*cmp)(const void*, const void *, const void *), void *arg);
+int cli_checktimelimit(cli_ctx *ctx);
 
 /* symlink behaviour */
 #define CLI_FTW_FOLLOW_FILE_SYMLINK 0x01
